/************************************************************************************
 * WrightEagle (Soccer Simulation League 2D)                                        *
 * BASE SOURCE CODE RELEASE 2009                                                    *
 * Copyright (C) 1998-2009 WrightEagle 2D Soccer Simulation Team,                   *
 *                         Multi-Agent Systems Lab.,                                *
 *                         School of Computer Science and Technology,               *
 *                         University of Science and Technology of China, China.    *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     *
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
 ************************************************************************************/

#ifndef __BASESTATE_H__
#define __BASESTATE_H__


#include "ServerParam.h"
#include "PlayerParam.h"
#include <cstring>

//ֵ
// Base value
template<class T>
class StateValue
{
public:
	StateValue():
		mValue (static_cast<T>(0)),
		mCycleDelay(9999),
		mConf(0)
	{
	}

	StateValue(const StateValue & o) {
		mValue = o.mValue;
		mCycleDelay = o.mCycleDelay;
		mConf = o.mConf;
	}

	/**ڲԸ
	* @param ÿdelayϵ
	* @param ÿconf˥*/
	/**Inner update
	* @param data added by delay
	* @param data decayed by conf*/
	void AutoUpdate(int delay_add = 1 , double conf_dec_factor = 1)
	{
		mCycleDelay += delay_add;
		mConf *= conf_dec_factor;
	}

public:
	/**ֵС*/
	/**Just a value*/
	T mValue;

	/**ֵ൱ǰʱ*/
	/**Time difference between current time and the last update time*/
	int mCycleDelay;

	/**ֵĿŶ*/
	/**T's confidence*/
	double mConf;
};

/////////////////////////////////////////////////////////////////////////////////////////////////////////
/**BaseState*/
/**Base of all state*/
class BaseState
{
public:
	BaseState()
	{
		mPos.mValue = Vector(10000 , 10000);
		mPosEps = 10000;
	}

	BaseState(const BaseState & o) {
		mPos = o.mPos;
	}

	/**λ
	* @param λ
	* @param ʱ
	* @param Ŷ*/
	/**Update position
	* @param position
	* @param time
	* @param confidence*/
	void UpdatePos(const Vector & pos , int delay = 0, double conf = 1);

	/**õǰǰελ
	* @param ֵ֮ǰΧΪ0RECORDSIZE-1,ѭ
	*/
	/**Get position*/
	const Vector & GetPos() const { return mPos.mValue; }

	/**õǰǰεʱ
	* @param ֵ֮ǰΧΪ0RECORDSIZE-1,ѭ
	*/
	/**Get position delay*/
	int GetPosDelay() const { return mPos.mCycleDelay; }

	/**õǰǰεŶ
	* @param ֵ֮ǰΧΪ0RECORDSIZE-1,ѭ
	*/
	/**Get position confidence*/
	const double & GetPosConf() const { return mPos.mConf; }


	/**
	 *  posΧ,ҪָԲİ뾶
	 */
	/**Set position's error band, radius of error circle*/
	void UpdatePosEps(double eps)  { mPosEps = eps;}

	/**
	 * poseps
	 */
	/**Get position's eps*/
	const double & GetPosEps() const { return mPosEps;}

	/**ڲԸ
	* @param ÿdelayϵ
	* @param ÿconf˥*/
	/**Inner update
	* @param data added by delay
	* @param data decayed by conf*/
	void AutoUpdate(int delay_add = 1 , double conf_dec_factor = 1);
private:
	/**洢ڵλ*/
	/**Save historical positions*/
	StateValue<Vector> mPos;

	double mPosEps;
};

//////////////////////////////////////////////////////////////////////////////////////////////////////
/**̬*/
/**עʱû뵽ʲôϢ*/
/**Static object's base state*/
/**Reserved*/
class StaticState: public BaseState
{
};

/**Mobile object's base state*/
class MobileState: public BaseState
{
public:
	MobileState(double decay, double effective_speed_max):
		mDecay(decay),
		mEffectiveSpeedMax(effective_speed_max),
		mpPredictor(new Predictor(mDecay)),
		mGuessed (0)
	{
		mVelEps = 10000;
		ResetPredictor();
	}

    MobileState(const MobileState &m):
    	BaseState(m),
    	mDecay(m.GetDecay()),
    	mEffectiveSpeedMax(m.GetEffectiveSpeedMax()),
    	mpPredictor(new Predictor(mDecay)),
    	mGuessed (0)
    {
		mVel = m.mVel;
		mVelEps = m.mVelEps;
		UpdatePos(m.GetPos(), m.GetPosDelay(), m.GetPosConf());
	}

	virtual ~MobileState() { delete mpPredictor; }

	void UpdatePos(const Vector & pos , int delay = 0, double conf = 1.0);

	const MobileState & operator=(const MobileState &v)
	{
		mVel = v.mVel;
		mDecay = v.mDecay;

		UpdatePos(v.GetPos(), v.GetPosDelay(), v.GetPosConf()); //use old predictor

		mEffectiveSpeedMax = v.mEffectiveSpeedMax;

		return *this;
	}

	/**ٶ
	* @param ٶ
	* @param ʱ
	* @param Ŷ*/
	/**Update velocity
	* @param velocity
	* @param time
	* @param confidence*/
	void UpdateVel(const Vector & vel , int delay = 0, double conf = 1);

	/**õǰǰεٶ
	* @param ֵ֮ǰΧΪ0RECORDSIZE-1,ѭ
	*/
	/**Get velocity*/
	const Vector & GetVel() const { return mVel.mValue; }

	/**õǰǰεٶȵʱ
	* @param ֵ֮ǰΧΪ0RECORDSIZE-1,ѭ
	*/
	/**Get velocity delay*/
	int GetVelDelay() const { return mVel.mCycleDelay; }

	/**õǰǰεٶȵŶ
	* @param ֵ֮ǰΧΪ0RECORDSIZE-1,ѭ
	*/
	/**Get velocity's confidence*/
	const double & GetVelConf() const { return mVel.mConf; }

	/**
	 * ٶ
	 * Update velocity's eps
	 */
	void UpdateVelEps(double eps) { mVelEps = eps;}

	/**
	 * ٶ
	 * Get velocity's eps
	 */
	double GetVelEps() const { return mVelEps;};

	/**
	* õ˶ decay
	* Get decay of object's motion
	* @return
	*/
	double GetDecay() const { return mDecay; }

	void SetDecay(double decay) { mDecay = decay; }

	/**
	* ʵٶ
	* Actual max speed
	*/
	const double & GetEffectiveSpeedMax() const { return mEffectiveSpeedMax; }
	void SetEffectiveSpeedMax(double effective_speed_max) { mEffectiveSpeedMax = effective_speed_max; }

	/**ڲԸ
	* @param ÿdelayϵ
	* @param ÿconf˥*/
	/**Inner update
	* @param data added by delay
	* @param data decayed by conf*/
	void AutoUpdate(int delay_add = 1 , double conf_dec_factor = 1);

private:
	class Predictor {
		const double & mDecay;

		static const int MAX_STEP = 50;

		Vector mPredictedPos[MAX_STEP + 1];
		Vector mPredictedVel[MAX_STEP + 1];
		int mPredictedStep;

	public:
		Predictor(const double & decay): mDecay(decay) {
		}

		const Vector & GetPredictedPos(int step = 1){
			Assert(step >= 0);
			step = Min(step, MAX_STEP);

			if (step <= mPredictedStep){
				return mPredictedPos[step];
			}
			else {
				PredictTo(step);

				return mPredictedPos[step];
			}
		}	
		const Vector & GetPredictedVel(int step = 1){
			Assert(step >= 0);
			step = Min(step, MAX_STEP);

			if (step <= mPredictedStep){
				return mPredictedVel[step];
			}
			else {
				PredictTo(step);

				return mPredictedVel[step];
			}
		}

		void UpdatePosAndVel(const Vector & pos, const Vector & vel) {
			mPredictedPos[0] = pos;
			mPredictedVel[0] = vel;
			mPredictedStep = 0;
		}

	private:
		void PredictTo(int step) {
			Assert (step >= 0 && step <= MAX_STEP);

			for (int i = mPredictedStep + 1; i <= step; ++i){
				mPredictedPos[i] = mPredictedPos[i-1] + mPredictedVel[i-1];
				mPredictedVel[i] = mPredictedVel[i-1] * mDecay;
			}
			mPredictedStep = step;
		}
	};

	void ResetPredictor() {
		mpPredictor->UpdatePosAndVel(GetPos(), GetVel());
	}

public:
	const Vector & GetPredictedPos(int step = 1) const {
		return mpPredictor->GetPredictedPos(step);
	}
	const Vector & GetPredictedVel(int step = 1) const {
		return mpPredictor->GetPredictedVel(step);
	}
//guessed time is now only for WorldStateUpdater , other should not use it;
	/** set guessed times*/
	void UpdateGuessedTimes(int guessed)
	{
		Assert(guessed >= 0);
		mGuessed = guessed;
	}

	/** get guessed times*/
	int GetGuessedTimes() const
	{
		return mGuessed;
	}

private:
	/**洢ڵٶ*/
	/**Save historical velocity*/
	StateValue<Vector> mVel;

	double mVelEps;

	double mDecay;
	double mEffectiveSpeedMax;

private:
	Predictor *mpPredictor;
	int mGuessed; //indicate that the times of this object ot be guessed -- only used from WorldStateUpdater::Run
};
#endif

