/************************************************************************************
 * WrightEagle (Soccer Simulation League 2D)                                        *
 * BASE SOURCE CODE RELEASE 2009                                                    *
 * Copyright (C) 1998-2009 WrightEagle 2D Soccer Simulation Team,                   *
 *                         Multi-Agent Systems Lab.,                                *
 *                         School of Computer Science and Technology,               *
 *                         University of Science and Technology of China, China.    *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     *
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
 ************************************************************************************/

#ifndef NOISEMODEL_H_
#define NOISEMODEL_H_

#include "Geometry.h"

/**
 * Noise model.
 */
class NoiseModel {
	NoiseModel();

	static const int VEL_SIZE = 300;	///ٶֵ0.013.0
	static const int CYCLE_SIZE = 50;	///049

    /**
     * 5ͨMatlabֵģKickTestĵ
     * The 5 following tables were calculated offline. See documents about KickTest for details.
     */
	double mX[VEL_SIZE][CYCLE_SIZE];    // xľֵ
	double mXs[VEL_SIZE][CYCLE_SIZE];	// xķ
	double mY[VEL_SIZE][CYCLE_SIZE];    // yľֵ
	double mYs[VEL_SIZE][CYCLE_SIZE];   // yķ
	double mPxy[VEL_SIZE][CYCLE_SIZE];  // ϵ

	double mMinBallSpeed;
	double mMaxBallSpeed;
public:
	static NoiseModel &instance();
	virtual ~NoiseModel();

	/**
	 * ȡļݣ浽5
	 * Read files and store them to 5 tables.
	 */
	bool ReadFile(const char* file_name, int index);

	/**
	 * ٴСõӦ±
	 * Get index from ball speed.
	 */
	int GetIndexFromSpeed(double speed);

	/**
	 * ٴС;õܶں󵽴õ
	 * Get cycles in need to a certain point most likely according to speed and distance.
	 */
	int GetBestCycle(double speed, double dis);

	/**
	 * ٴС;õܶں󵽴õ
	 * Get cycles in need to a certain point most likely according to speed and distance.
	 */
	void GetMoreVelocity(Vector ball_vel, double max_rand, Vector vel[]);

	/**
	 * õĳһĸܶ
	 * Get probability density on a certain point.
	 */
	double GetPDF(int vel_index, int cyc_index, Vector pos);

	/**
	 * ±õĳһΪԲĵԲĸʣػʱԲھȡ17
	 * Get probability on a circle centered at a certain point. Checked 17 points when calculating double
	 * integration.
	 */
	double GetPossibilityFromIndex(int vel_index, int cyc_index,  Vector pos0, double r);

	/**
	 * ɳٶȺ;ڵõĳһΪԲĵԲĸ
	 * Get probability on a circle centered at a certain point.
	 */
	double GetPossibilityFromSpeed(double speed, int cycle, Vector goal_pos, double r);

	/**
	 * õĳһΪԲĵԲĸ
	 * Get probability on a circle centered at a certain point.
	 */
	double GetPossibility(Vector ball_vel, double max_rand, int cycle, Vector pos_2_ball, double r);

	/**
	 * õxyıԵܶ
	 * Get marginal probability density for x or y.
	 */
	double GetPDFXY(int vel_index, int cyc_index, double param, bool is_x);

	/**
	 * õĳһĸʣnewton-cotesʽ
	 * Get probability on (low, high) through newton-cotes formula.
	 */
	double GetPXYFromIndex(int vel_index, int cyc_index, double low, double high, bool is_x);

	/**
	 * õĳһĸ
	 * Get probability on (-, high)
	 */
	double GetPXYFromIndex(int vel_index, int cyc_index, double high, bool is_x);

	/**
	 * ٶֵõĳһĸ
	 * Get probability on (-, high) via speed.
	 */
	double GetPXYFromSpeed(double speed, int cycle, double high, bool is_x);

	/**
	 * õĳһxyֵĸ
	 * Get probability on interval from - to a certain point.
	 */
	double GetPossibilityXY(Vector ball_vel, double max_rand, int cycle, Vector pos_2_ball, bool is_x);

	/**
	 * ٶںϷٶȷΧڣ0.01 - 3.0
	 * Normalize give speed to legal range.
	 */
	void NormalizeSpeed(double& speed);

	/**
	 * ֵںϷΧ
	 * Normalize indexes to legal range.
	 */
	void NormalizeIndex(int& vel_index, int& cyc_index);
};

#endif /* NOISEMODEL_H_ */
