/************************************************************************************
 * WrightEagle (Soccer Simulation League 2D)                                        *
 * BASE SOURCE CODE RELEASE 2009                                                    *
 * Copyright (C) 1998-2009 WrightEagle 2D Soccer Simulation Team,                   *
 *                         Multi-Agent Systems Lab.,                                *
 *                         School of Computer Science and Technology,               *
 *                         University of Science and Technology of China, China.    *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     *
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
 ************************************************************************************/

#ifndef PARAMENGINE_H_
#define PARAMENGINE_H_

#include <string>
#include <list>
#include <iostream>

/**
 * ͣ
 * Ϊ˼ȥ V_BOOL ͣV_BOOL ʾѡͲģ
 *  -name ʾ name Ϊ true -goalie, -dd,
 * ҪʵͬĹΪ -goalie on, -dd on
 */
enum ParamType {
	V_NONE,

	V_INT,		//
	V_DOUBLE,	//ʵ
	V_STRING,	//ַ -- ֻ std::string
	V_ONOFF		//
};

template < typename T >
struct ParamTypes {
	static const ParamType TYPE = V_NONE;
};

template <>
struct ParamTypes <int>
{
	static const ParamType TYPE = V_INT;
};

template <>
struct ParamTypes <double>
{
	static const ParamType TYPE = V_DOUBLE;
};

template <>
struct ParamTypes <std::string>
{
	static const ParamType TYPE = V_STRING;
};

template <>
struct ParamTypes <bool>
{
	static const ParamType TYPE = V_ONOFF;
};

//Parameter structure
struct Param {
	std::string name;
	void *ptr;        //pointer to data field
	ParamType type;

	Param(const char *str, void *p, ParamType t):
		name(str),
		ptr(p),
		type(t)
	{
	}
};

/**
 * Ҫֵ/serverϢ/ļ;
 */
/**
 * This class is mainly used to assign parameters, including params from console, server 
 * messages and configure files.
 */
class ParamEngine {
public:
	typedef std::list<Param>::iterator ParamPtr;

private:
	static const int HASH_SIZE = 512;
	typedef std::list<Param> ParamList;

	ParamList mParamLists[HASH_SIZE];

	/**
	 * Get the hash of a certain string.
	 * @param str the string.
	 * @return the hash of the string.
	 */
	int hash(const char *str);

	/**
	 * This funtion is to check if the two string is equal.
	 * @param a string a.
	 * @param b string b.
	 * @return true if string a == string b.
	 */
	bool str_equal(const char *a, const char *b);

public:
	ParamEngine() {}
	virtual ~ParamEngine() {}

	/**
	 * Add params with some values.
	 * @param name the name of the param.
	 * @param address the address of the param.
	 * @param value the value of the param to assign with.
	 */
	template <typename T>
	void AddParam(const char *name, T *address, T value){
		if (address != NULL){
			// assign default value
			*address = value;

			// allocate Param struct
			Param param(name, static_cast<void*>(address), ParamTypes<T>::TYPE);

			if (param.type == V_NONE) {
				std::cerr << __FILE__ << ":" << __LINE__ << ": param type error \"" << name << "\"" << std::endl;
				return;
			}
			// put into list
			int id = hash(name);
			if (id > -1){
				this->mParamLists[id].push_back(param);
			}
		}
	}

	/**
	 * Set the params with values.
	 * @param name the name of the param.
	 * @param value the value of the param to assign with.
	 * @return true means the setting done successfully.
	 */
	template <typename T>
	bool SetParam(const char *name, T value)
	{
		if (name != NULL ){
			ParamPtr it;
			bool found = GetParam(name, it);
			if (found == true){
				switch(it->type){
					case V_INT:
					case V_DOUBLE:
					case V_STRING:
					case V_ONOFF:
						*static_cast<T*>(it->ptr) = value;
						break;
					default:
						return false;
				}
				return true;
			}
			else {
				std::cerr << __FILE__ << ":" << __LINE__ << ": unknown argument \'" << name << "\'" << std::endl;
			}
		}
		return false;
	}

	virtual void AddParams() = 0;

	/**
	 * Set params from string.
	 * @param name the name of the param.
	 * @param buffer the string.
	 * @return true means the setting done successfully.
	 */
	bool SetParamFromString(const char *name, const char *buffer);

	/**
	 * Get the param iterator.
	 * @param name the name of the param.
	 * @param it the iterator pointer to return.
	 * @return true means the param found successfully.
	 */
	bool GetParam(const char *name, ParamPtr & it);

	/**
	 * Print the param.
	 * @param name the name of the param.
	 * @param os ostream of std.
	 * @return true means the param printed successfully.
	 */
	bool PrintParam(const char *name, std::ostream &os);

	/**
	 * Print the param.
	 * @param name the name of the param.
	 * @return true means the param printed successfully.
	 */
	bool PrintParam(const char *name);

	/**
	 * Dump the params.
	 * @param os ostream of std.
	 */
	void DumpParam(std::ostream &os);

	/**
	 * Dump the params.
	 */
	void DumpParam();

	/**
	 * Save the params to a config file.
	 * @param file_name the file name of the config file.
	 * @return true means the file has been written successfully.
	 */
	bool SaveToConfigFile(const char *file_name);

	/**
	 * ȡϢʽ-name value ...
	 * @param argc 
	 * @param argv ַ׵ַ
	 */
	/**
	 * Parse params from command line.
	 * @param argc number of params.
	 * @param argv the point to the param strings' point.
	 */
	void ParseFromCmdLine(int argc, char **argv);

	/**
	 * serverϢȡϢʽ(name value) ...
	 * @param line serverϢԭ
	 */
	/**
	 * Parse params from server message.
	 * @param line the raw message from server.
	 */
	void ParseFromServerMsg(const char *line);

	/**
	 * ļȡϢʽname = value
	 * @param file_name
	 * @return Ƿɹȡ
	 */
	/**
	 * Parse params from config file.
	 * @param file_name the name fo the config file.
	 * @return true means the params have been read from the config file successfully.
	 */
	bool ParseFromConfigFile(const char *file_name);
};

#endif /* PARAMENGINE_H_ */
