/************************************************************************************
 * WrightEagle (Soccer Simulation League 2D)                                        *
 * BASE SOURCE CODE RELEASE 2009                                                    *
 * Copyright (C) 1998-2009 WrightEagle 2D Soccer Simulation Team,                   *
 *                         Multi-Agent Systems Lab.,                                *
 *                         School of Computer Science and Technology,               *
 *                         University of Science and Technology of China, China.    *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     *
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
 ************************************************************************************/

#include "Player.h"
#include "DecisionTree.h"
#include "DynamicDebug.h"
#include "Formation.h"
#include "CommandSender.h"
#include "Parser.h"
#include "Thread.h"
#include "UDPSocket.h"
#include "WorldModel.h"
#include "Agent.h"
#include "Logger.h"
#include "CommunicateSystem.h"
#include "TimeTest.h"

//==============================================================================
Player::Player():
	mpDecisionTree( new DecisionTree )
{
}

//==============================================================================
Player::~Player()
{
	delete mpDecisionTree;
}

/**
 * Send player options to the server.
 */
void Player::SendOptionToServer()
{
	while (!mpParser->IsClangOk())
	{
		mpAgent->CheckCommands(mpObserver);
		mpAgent->Clang(7, 8);
		mpObserver->SetCommandSend();
		WaitFor(200);
	}

	while (!mpParser->IsSyncOk())
	{
		mpAgent->CheckCommands(mpObserver);
		mpAgent->SynchSee();
		mpObserver->SetCommandSend();
		WaitFor(200);
	}

	mpAgent->CheckCommands(mpObserver);
	mpAgent->EarOff(false);
	mpObserver->SetCommandSend();
	WaitFor(200);
}

/**
 * Main loop of player.
 */
void Player::Run()
{
    //TIMETEST("Run");

	static Time last_time = Time(-100, 0);

	mpObserver->Lock();

	/** 漸˳ܱ */
	CommunicateSystem::instance().Update(); // hearϢȸ
	mpAgent->CheckCommands(mpObserver);
	mpWorldModel->Update(mpObserver);
    mpAgent->GetFormation().UpdateOpponentRole(); // TODO: ʱδϢʱԼȼ

	mpObserver->UnLock();

    const Time & time = mpAgent->GetWorldState().CurrentTime();

	if (last_time.T() >= 0) {
		if (time != Time(last_time.T() + 1, 0) && time != Time(last_time.T(), last_time.S() + 1)) {
			if (time == last_time) {
				mpAgent->World().SetCurrentTime(Time(last_time.T(), last_time.S() + 1)); //ݸ»
			}
		}
	}

	last_time = time;

	if (mpAgent->World().Ball().GetPosConf() < 0.93)
	{
		mpAgent->Turn(60);
	}
	else
	{
		mpDecisionTree->Decision(*mpAgent);
		CommunicateSystem::instance().Decision();


		{
			double body_dir = mpAgent->GetSelfBodyDirWithQueuedActions();
			if (mpAgent->GetSelf().GetViewWidth() != VW_Narrow) mpAgent->ChangeView(VW_Narrow);
			mpAgent->TurnNeck(
				GetNormalizeAngleDeg(
				(mpAgent->World().Ball().GetPos() - mpAgent->GetSelf().GetPos()).Dir() 
				- (mpAgent->GetSelf().GetNeckDir() + body_dir)));
		}
		/*if (mpAgent->GetSelfUnum() == 11)
		{
			std::cout << mpAgent->World().CurrentTime() << std::endl; 
			std::cout << body_dir << std::endl;
			std::cout << mpAgent->GetSelf().GetNeckDir() << std::endl;
			std::cout << mpAgent->GetSelf().GetBodyDir() << std::endl;
		}*/
	}

	Logger::instance().LogSight();
}


//end of file Player.cpp
