/************************************************************************************
 * WrightEagle (Soccer Simulation League 2D)                                        *
 * BASE SOURCE CODE RELEASE 2009                                                    *
 * Copyright (C) 1998-2009 WrightEagle 2D Soccer Simulation Team,                   *
 *                         Multi-Agent Systems Lab.,                                *
 *                         School of Computer Science and Technology,               *
 *                         University of Science and Technology of China, China.    *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     *
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
 ************************************************************************************/

#ifndef __WORLDSTATE_H__
#define __WORLDSTATE_H__

#include <vector>
#include <cstdlib>
#include "Observer.h"
#include "PlayerState.h"
#include "BallState.h"
#include "HearMessage.h"
#include "CommunicateSystem.h"

class PlayerObserver;
class UnknownPlayerObserver;
class UnknownUnumPlayerObserver;

const int HISTORYSIZE = 10;

/**
 * 泡״̬Ϣ
 */
/**
 * Save history information of current game
 */
class HistoryState;

/**
 * WorldState ϢԼú޹أϢöѺͶ֣ԱҲԼı޹
 */
/**
 * WorldState 
 * player's number in WorldState: > 0 -> teammate ; < 0 -> opponent
 * our goal's X() always < 0
 */
class WorldState {
    friend class WorldStateUpdater;
    WorldState(const WorldState &);

public:
    WorldState(HistoryState * history_state = 0);
    virtual ~WorldState() {}

	/**
	 * ObserverWorldState
	 * @param observer
	 */
	/**
	 * Update from Observer
	 * @param observer
	 */
    void UpdateFromObserver(Observer *observer);

	/**
	 * world_state£õһʱõ״̬
	 * @param world_state
	 */
	/**
	 * Update from world_state to get a world state for calculating opponent's planning
	 * @param world_state
	 */
    void GetReverseFrom(WorldState *world_state);

    const PlayerState & GetPlayer(Unum i) const { Assert(abs(i) >= 1 && abs(i) <= TEAMSIZE); return i > 0? mTeammate[i]: mOpponent[-i]; }
    const std::vector<PlayerState *> & GetPlayerList() const { return mPlayerList; }

    const BallState & GetBall() const { return mBall; }
    const Time & CurrentTime() const { return mCurrentTime; }
	void  SetCurrentTime(const Time & time) { mCurrentTime = time; }
	KickOffMode GetKickOffMode() const {return mKickOffMode;}

    PlayMode GetPlayMode() const { return mPlayMode; }
	PlayMode GetLastPlayMode() const { return mLastPlayMode;}
    const Time & GetPlayModeTime() const { return mPlayModeTime; }

	BallState & Ball() { return mBall; }
	/**
	 * Opponent's num < 0
	 * Teammate's num > 0
	 * 0 isn't available
	 */
	PlayerState & Player(Unum i) { Assert(abs(i) >= 1 && abs(i) <= TEAMSIZE); return i > 0? mTeammate[i]: mOpponent[-i]; }
    PlayerState & Teammate(Unum i) { Assert(i >= 1 && i <= TEAMSIZE); return mTeammate[i]; }
    PlayerState & Opponent(Unum i) { Assert(i >= 1 && i <= TEAMSIZE); return mOpponent[i]; }

    const PlayerState & GetTeammate(Unum i) const { if(!(PlayerParam::instance().isCoach() && i == 0)) { Assert(i >= 1 && i <= TEAMSIZE); } return mTeammate[i]; }
    const PlayerState & GetOpponent(Unum i) const { Assert(i >= 1 && i <= TEAMSIZE); return mOpponent[i]; }

    Unum GetTeammateGoalieUnum() const { return mTeammateGoalieUnum; }
    Unum GetOpponentGoalieUnum() const { return mOpponentGoalieUnum; }

    int GetTeammateScore() const { return mTeammateScore; }
    int GetOpponentScore() const { return mOpponentScore; }

	/**
	 * get record history
	 * @param i stand for i cycles before ,if i = 0 ,it means that you get current worldstate
	 */
	WorldState * GetHistory(int i) const;

	/**
	 * get time cycle  before current time
	 * warning : this should not be used when you get WorldState from history  for example: GetHistory(1)->GetTimeBeforeCurrent(1) is not correct
	 * @param cycle should be >= 1 and <= HISTORYSIZE
	 */
	Time GetTimeBeforeCurrent(int cycle = 1) const;

	bool IsBallDropped() const { return mIsBallDropped; }

	//set play cycle
	//cycle for set play
	const Time& GetSetPlayHearTime() const { return mSetPlayHearTime;}
	const int& GetSetPlayStaminaCycle() const { return mSetPlayStaminaCycle;}
	const int& GetSetPlayCapacityCycle() const { return mSetPlayCapacityCycle;}

	const PassInfo & GetHearPassInfo() const { return mHearMessage.GetHearPassInfo(); }
	const PassInfo & GetAskForBallInfo() const { return mHearMessage.GetAskForBallInfo(); }
	const TacticsInfo & GetTacticsInfo() const { return mHearMessage.GetTacticsInfo();}
	const BehaviorType & GetControllerType() const { return mHearMessage.GetTacticsInfo().mType;}
    const HearMessage & GetHearMessage() const { return mHearMessage; }
	HearMessage & HearMsg() { return mHearMessage; }

private:
	HistoryState * mpHistory;

    Time mCurrentTime;

	KickOffMode mKickOffMode;
    PlayMode mPlayMode;
	PlayMode mLastPlayMode;
	Time mPlayModeTime;
	bool mIsBallDropped;

    BallState mBall;
    PlayerState mTeammate[TEAMSIZE + 1];
    PlayerState mOpponent[TEAMSIZE + 1];
    std::vector<PlayerState *> mPlayerList;

    Unum mTeammateGoalieUnum;
    Unum mOpponentGoalieUnum;

	Time mSetPlayHearTime;
	int  mSetPlayStaminaCycle;
	int  mSetPlayCapacityCycle;

	int mTeammateScore;
	int mOpponentScore;

	bool mIsCycleStopped;
	HearMessage mHearMessage;		/** Used to save hear message */
};

/**
 * WorldStateĸ
 */
/**
 * WorldState Updater
 * Used to update WorldState only
 */
class WorldStateUpdater {
    WorldStateUpdater(const WorldStateUpdater &);

public:
    WorldStateUpdater(Observer *observer, WorldState *world_state):
    	mpObserver( observer ),
    	mpWorldState( world_state ),
    	mSelfSide( mpObserver? mpObserver->OurSide(): '?' ),
    	mSelfUnum( mpObserver? mpObserver->MyUnum(): 0 )
    {
		mBallConf = 1;
		mPlayerConf = 1;
		mSightDelay = 0;
		mIsHearBallPos = false;
		mIsHearBallVel = false;
    }

    virtual ~WorldStateUpdater() {}

     /**
      * 
      */
	/**
	 * Updating
	 */
    void Run();

private:
    /** һЩֻڸʱõĽӿ */
	/** Some interface for update only */
    inline BallState   & Ball();
    inline PlayerState & Teammate(Unum i);
    inline PlayerState & Opponent(Unum i);
    inline PlayerState & SelfState();

	/**methods*/
    inline const BallState & GetBall();
    inline const PlayerState & GetTeammate(Unum i);
    inline const PlayerState & GetOpponent(Unum i);
    inline const PlayerState & GetSelf();
	inline AngleDeg  GetNeckGlobalDirFromSightDelay(int sight_delay = 0);
	inline Vector    GetSelfVelFromSightDelay(int sight_delay = 0);
    inline Unum GetSelfUnum();
    inline char GetSelfSide();

private:
    /** ָ· */
	/** Update WorldState */
	void UpdateWorldState();

private:
    /**
     * õǰɼĶ
     * û򷵻صĺΪ0
     */
	/**
	 * Get Seen Closest Teammate
	 * return 0 if no teammate seen
	 */
    Unum GetSeenClosestTeammate();

    /**
     * õǰɼĶ
     * û򷵻صĺΪ0
     */
	/**
	 * Get Seen Closest Opponent
	 * return 0 if no opponent seen
	 */
    Unum GetSeenClosestOpponent();

	/**
	 * get closest player
	 *  just for EvaluatePlayer
	 */
	Unum GetClosestOpponent();

	/**
	 * get closest player to ball
	 * @return is teammate ? > 0 : < 0
	 */
	Unum GetClosestPlayerToBall();


//=======================================ڲµĽӿ================================================
//===================================Inner Update Interface===========================================
public:
    /** ActionҪϢ */
	/** Update Action Information */
    void UpdateActionInfo();

private:
	/**update sight delay */
	void UpdateSightDelay();

    /** ԼϢ */
	/** Update Self Information */
    void UpdateSelfInfo();

    /** Լsense */
	/** Update Self sense Information */
    void UpdateSelfSense();

    /** еĶԱϢ */
	/** Update All Players' Information */
    void UpdateKnownPlayers();

    /** δ֪ԱϢ */
	/** Update Unknown Players' Information */
    void UpdateUnknownPlayers();

	bool UpdateMostSimilarPlayer(Vector pos ,int index);

    /** ĳһضĶԱ */
	/** Update a Specific Player */
    void UpdateSpecificPlayer(const PlayerObserver& player , Unum unum , bool is_teammate);

    /** ĳضδ֪Ա */
	/** Update a Specific Unknown Player */
    void UpdateSpecificUnknownPlayer(const UnknownPlayerObserver& player , Unum num , bool is_teammate);

    /** Ϣ */
	/** Update Ball Information */
    void UpdateBallInfo();

    /** Զdelay1 */
	/** Auto Add Delay with 1 */
    void AutoDelay(int delay_add);

    /** ضλôض״̬ */
	/** Update a Specific Information from Play Mode */
    void UpdateInfoFromPlayMode();

    /** ³ϵϢ */
	/** Update Field Information */
    void UpdateFieldInfo();

    /**  */
	/** Update Information from Hear Information */
    void UpdateFromAudio();

	/** ݴϢȸ*/
	/** Update Information from Pass Information */
	void UpdateFromHearInfo();

    /** collide */
	/** Update Information about colliding */
    void UpdateInfoWhenCollided();
//=========================================ºԤļ㹤=============================================
//==================================Some Update and Forecast Tools========================================
private:
    /**ԼͷĽǶ*/
	/**Compute my head's direction*/
    bool ComputeSelfDir(double& angle);

    /**Լλ*/
	/**Compute my position*/
    bool ComputeSelfPos(Vector& vec , double& eps);

    /**һ*/
	/**Compute next cycle*/
    bool ComputeNextCycle(MobileState& ms , double decay);

    double ComputePlayerMaxDist(const PlayerState& state);

    /**compute the player may see or not. just for update unknown player */
    bool ComputePlayerMaySeeOrNot(const PlayerState& state);

	/**compute conf from cycle*/
	double ComputeConf(double delay , int cycle);

	/** estimate to now*/
	void EstimateToNow();

    /** ˶ɺִеĶģһ */
	/** EstimateWorld by action of last cycle */
    void EstimateWorld(bool is_estimate_to_now = false , int cycle = 0);

    /** ԤԼϢ */
	/** Estimate self information */
    void EstimateSelf(bool is_estimate_to_now = false , int cycle = 0);

    /** ԤϢ */
	/** Estimate ball information */
    void EstimateBall(bool is_estimate_to_now = false , int cycle = 0);

    /** ԤԼԱϢ */
	/** Estimate Teammates Information */
    void EstimatePlayers();

	/**
	 * recompute conf
	 */
	void EvaluateConf();

	/**
	 * Evaluate ball forgotten
	 */
	void EvaluateBall();

	/**
	 * evaluate forget ball
	 */
	void EvaluateForgetBall();

	/**
	 * evaluate player
	 */
	void EvaluatePlayer(PlayerState& player , bool is_teammate);

	/**
	 * evaluate to forget player
	 */
	void EvaluateForgetPlayer(PlayerState& player);

	/**
	 * calc player go to point
	 */
	double CalcPlayerGotoPoint(PlayerState& player ,double dist);

	/**
	 * tell whether can see or not
	 */
	bool ComputeShouldSeeOrNot(Vector pos);

	void MaintainPlayerStamina();

	void MaintainConsistency();

	void UpdateOtherKick();
private:
    Observer * const mpObserver;
    WorldState * const mpWorldState;

    char mSelfSide;
    Unum mSelfUnum;

	double mPlayerConf;
	double mBallConf;
	int mSightDelay;

	static const double KICKABLE_BUFFER;
	static const double CATCHABLE_BUFFER;

	bool  mIsOtherKick;
	int   mOtherKickUnum; // >0 : teammate , < 0 opponent;

	bool  mIsOtherMayKick;

	bool mIsHearBallPos;
	bool mIsHearBallVel;

public:
    /** ԱĲɹʣΪ£ԷȽϺ */
	/** Compute Tackle Probability */
    double ComputeTackleProb(const Unum & unum);
};

/**
 * worldstateݣԹѻָ֣״̬
 */
/**
 * EXAMPLE:
 * void Behavior*Planner::Plan() {
 *      ActiveBehavior teammate_behavior;
 *
 *      { //ָڵĴ뷶Χʹָ
 *      	WorldStateSetter setter(mWorldState);
 *      	setter.Ball().UpdatePos(Vector(0, 0));
 *      	//... ...
 *          setter.IncStopTime(); //Կʼ
 *      	Agent * agent = mAgent.CreateTeammateAgent(mStrategy.GetSureTm());
 *          ActiveBehaviorList bhv_list;
 *      	BehaviorPassPlanner(*agent).Plan(bhv_list);
 *          teammate_behavior = bhv_list.front();
 *          delete agent;
 *          //setterָ״̬
 *      }
 *
 *      //do_some_thing_about teammate_behavior
 *      //... ...
 * }
 *
 */
 /**
  * WorldStateSetter
  * Used to Set information in WorldState
  * mainly to calculate opponent's plan
  * recovered by ~WorldStateSetter()
  */
class WorldStateSetter {
public:
	WorldStateSetter (WorldState & world_state): mWorldState(world_state) {
		mBackupTime = mWorldState.CurrentTime();
		mpBackupBallState = 0;
		memset(mpBackupPlayerState, 0, sizeof(mpBackupPlayerState));
	}

	~WorldStateSetter() {
		mWorldState.SetCurrentTime(mBackupTime);
		if (mpBackupBallState != 0) {
			mWorldState.Ball() = *mpBackupBallState;
			delete mpBackupBallState;
		}
		for (int i = 0; i < TEAMSIZE * 2 + 1; ++i) {
			if (mpBackupPlayerState[i] != 0) {	//ݹˣڻָ
												//recovered
				Assert(i != 0);
				if (i <= TEAMSIZE) {
					mWorldState.Teammate(i) = *mpBackupPlayerState[i];
					delete mpBackupPlayerState[i];
				}
				else {
					mWorldState.Opponent(i - TEAMSIZE) = *mpBackupPlayerState[i];
					delete mpBackupPlayerState[i];
				}
			}
		}
//		WorldStateUpdater(0, & mWorldState).UpdateActionInfo();
	}

	/**
	 * ҪĽӿstateݣܱ
	 * @return
	 */
	/**
	 * The following functions are used to change data in state
	 * @return
	 */
	BallState   & Ball() { if (pBall() == 0) pBall() = new BallState(mWorldState.Ball()); return mWorldState.Ball(); }
	PlayerState & Teammate(Unum i) { if (pTeammate(i) == 0) pTeammate(i) = new PlayerState(mWorldState.Teammate(i)); return mWorldState.Teammate(i); }
	PlayerState & Opponent(Unum i) { if (pOpponent(i) == 0) pOpponent(i) = new PlayerState(mWorldState.Opponent(i)); return mWorldState.Opponent(i); }

	void SetBallInfo(Vector Pos, Vector Speed)
	{
		Ball().UpdatePos(Pos);
		Ball().UpdateVel(Speed);
	}
	void SetTeammateInfo(Unum num,Vector Pos, AngleDeg BodyAngle, Vector Speed)
	{
		Teammate(num).UpdatePos(Pos);
		Teammate(num).UpdateBodyDir(BodyAngle);
		Teammate(num).UpdateVel(Speed);
	}
	void SetOpponentInfo(Unum num,Vector Pos, AngleDeg BodyAngle, Vector Speed)
	{
			Opponent(num).UpdatePos(Pos);
			Opponent(num).UpdateBodyDir(BodyAngle);
			Opponent(num).UpdateVel(Speed);
	}
	void IncStopTime() { WorldStateUpdater(0, & mWorldState).UpdateActionInfo(); mWorldState.SetCurrentTime(Time(mWorldState.CurrentTime().T(), mWorldState.CurrentTime().S() + 1)); }

private:
	BallState   *& pBall() { return mpBackupBallState; }
	PlayerState *& pTeammate(Unum i) { Assert(i >= 1 && i <= TEAMSIZE); return mpBackupPlayerState[i]; }
	PlayerState *& pOpponent(Unum i) { Assert(i >= 1 && i <= TEAMSIZE); return mpBackupPlayerState[TEAMSIZE + i]; }

private:
	WorldState & mWorldState;

	BallState   * mpBackupBallState;
	PlayerState * mpBackupPlayerState[1 + TEAMSIZE * 2];
	Time mBackupTime;
};

/**¼StateWorldʷϢ*/
/**Record History of WorldState*/
class HistoryState
{
public:
    HistoryState()
    {
        mNum = 0;
    }

    /**ǰ
     * @param ʵ
     */
	/**
	 * Push back current world state
	 * @param current world state
	 */
    void UpdateHistory(const WorldState & world);

    /**֮ǰ
     * @param ȡֵΧΪ1~HISTORYSIZEµǰ
     */
	/**
	 * Get history world state
	 * @param num 1~HISTORYSIZE
	 * @return worldstate
	 */
    WorldState *GetHistory(int num);
private:
    /**¼StateWorld*/
	/**record world state*/
    WorldState mRecord[HISTORYSIZE];

    /**¼鵱ǰöǰһհ*/
	/**record a space before current array of world state*/
    int mNum;
};

#endif /* WORLDSTATE_H_ */
