/************************************************************************************
 * WrightEagle (Soccer Simulation League 2D)                                        *
 * BASE SOURCE CODE RELEASE 2009                                                    *
 * Copyright (C) 1998-2009 WrightEagle 2D Soccer Simulation Team,                   *
 *                         Multi-Agent Systems Lab.,                                *
 *                         School of Computer Science and Technology,               *
 *                         University of Science and Technology of China, China.    *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     *
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
 ************************************************************************************/

#include <iostream>
#include <cstdlib>
#include <string>
#include <cstring>
#include <sstream>
#include "Coach.h"
#include "Player.h"
#include "ServerParam.h"
#include "PlayerParam.h"
#include "Logger.h"
#include "DynamicDebug.h"

#ifndef WIN32
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#endif

namespace {
#ifndef WIN32
void create_dump(void)
{
	if(!fork()) {
		abort(); //cause to send SIGABRT to itself, a core dump is generated by default
	}
}
#endif

void signal_catcher(int sig)
{
#ifndef WIN32
	signal(sig, signal_catcher);

	std::cerr << "catch signal " << sig << std::endl;

	create_dump(); //Ȳcoredump

	std::cerr << "exiting safely now" << std::endl;

	Logger::instance().LogSight();
	Logger::instance().SetFlushCond(); // set flush cond and let the logger thread flush the logs to file.
	DynamicDebug::instance().AddMessage("---Aborted", MT_Send);

	WaitFor(100); // wait logger thread to finish flushing

	exit(0); //þ̬
#endif
}
}

void RegisterSignalHandler();

//==============================================================================
int main(int argc, char* argv[])
{
	RegisterSignalHandler();

	ServerParam::instance().init(argc, argv);
	PlayerParam::instance().init(argc, argv);

	if (PlayerParam::instance().isCoach() == true) //coach
	{
		Coach *coach = new Coach;
		if (PlayerParam::instance().DynamicDebugMode() == true)
		{
			coach->RunDynamicDebug(); // 붯̬ģʽ
		}
		else
		{
			coach->RunNormal(); // ģʽ
		}
		delete coach;
	}
	else //player
	{
		Player *player = new Player;

		if (PlayerParam::instance().DynamicDebugMode() == true)
		{
			player->RunDynamicDebug(); // 붯̬ģʽ
		}
		else
		{
			player->RunNormal(); // ģʽ
		}

		delete player;
	}

	return 0;
}


void RegisterSignalHandler()
{
#ifndef WIN32
	if(signal(SIGINT, signal_catcher) == SIG_ERR) //ж
		perror("SIGINT"), exit(1);
	if(signal(SIGQUIT, signal_catcher) == SIG_ERR) //˳
		perror("SIGQUIT"), exit(1);
//	if(signal(SIGABRT, signal_catcher) == SIG_ERR) //עSIGABRT abort ʱѭ
//		perror("SIGABRT"), exit(1);
	if(signal(SIGSEGV, signal_catcher) == SIG_ERR) //δ
		perror("SIGSEGV"), exit(1);
	if(signal(SIGILL, signal_catcher) == SIG_ERR) //Ƿָ
		perror("SIGILL"), exit(1);
	if(signal(SIGFPE, signal_catcher) == SIG_ERR) //쳣
		perror("SIGFPE"), exit(1);
#endif
}
